<?php
/*
Plugin Name: External file
Plugin URI: http://adamhopkinson.co.uk
Description: External file allows a post author to include the contents of a file or url into a post or page. The included content will be highlighted based on the file extension or mime type. Highlighting is done by Alex Gorbatchevs' SyntaxHighlighter (http://alexgorbatchev.com). 
Author: Adam Hopkinson
Version: 0.1
Author URI: http://adamhopkinson.co.uk
*/

$plugin_dir = dirname(__FILE__);
$plugin_url = get_bloginfo('wpurl') . '/wp-content/plugins/external';

$formats = array(
	'xhtml',
	'html',
	'php'
);

/*
function external_file_highlight()
{
	$plugin_dir = dirname(__FILE__);
	echo $plugin_dir;
	return $plugin_dir;
}
*/

function external_header() {
	
	global $plugin_url;
	
	wp_enqueue_style('syntaxhighlighter-core', $plugin_url . '/highlighter/styles/shCore.css');
	wp_enqueue_style('syntaxhighlighter-theme', $plugin_url . '/highlighter/styles/shThemeDefault.css');

}

function external_scripts() {
	
	global $plugin_url;
	
	if(!is_admin()) {
		wp_enqueue_script('syntaxhighlighter-core', $plugin_url . '/highlighter/src/shCore.js');
		wp_enqueue_script('syntaxhighlighter-brush-as3', $plugin_url . '/highlighter/scripts/shBrushAS3.js');
		wp_enqueue_script('syntaxhighlighter-brush-bash', $plugin_url . '/highlighter/scripts/shBrushBash.js');
		wp_enqueue_script('syntaxhighlighter-brush-Cpp', $plugin_url . '/highlighter/scripts/shBrushCpp.js');
		wp_enqueue_script('syntaxhighlighter-brush-CSharp', $plugin_url . '/highlighter/scripts/shBrushCSharp.js');
		wp_enqueue_script('syntaxhighlighter-brush-Css', $plugin_url . '/highlighter/scripts/shBrushCss.js');
		wp_enqueue_script('syntaxhighlighter-brush-Delphi', $plugin_url . '/highlighter/scripts/shBrushDelphi.js');
		wp_enqueue_script('syntaxhighlighter-brush-Diff', $plugin_url . '/highlighter/scripts/shBrushDiff.js');
		wp_enqueue_script('syntaxhighlighter-brush-Groovy', $plugin_url . '/highlighter/scripts/shBrushGroovy.js');
		wp_enqueue_script('syntaxhighlighter-brush-Java', $plugin_url . '/highlighter/scripts/shBrushJava.js');
		wp_enqueue_script('syntaxhighlighter-brush-JavaFX', $plugin_url . '/highlighter/scripts/shBrushJavaFX.js');
		wp_enqueue_script('syntaxhighlighter-brush-JScript', $plugin_url . '/highlighter/scripts/shBrushJScript.js');
		wp_enqueue_script('syntaxhighlighter-brush-perl', $plugin_url . '/highlighter/scripts/shBrushPerl.js');
		wp_enqueue_script('syntaxhighlighter-brush-php', $plugin_url . '/highlighter/scripts/shBrushPhp.js');
		wp_enqueue_script('syntaxhighlighter-brush-plain', $plugin_url . '/highlighter/scripts/shBrushPlain.js');
		wp_enqueue_script('syntaxhighlighter-brush-powershell', $plugin_url . '/highlighter/scripts/shBrushPowerShell.js');
		wp_enqueue_script('syntaxhighlighter-brush-python', $plugin_url . '/highlighter/scripts/shBrushPython.js');
		wp_enqueue_script('syntaxhighlighter-brush-ruby', $plugin_url . '/highlighter/scripts/shBrushRuby.js');
		wp_enqueue_script('syntaxhighlighter-brush-scala', $plugin_url . '/highlighter/scripts/shBrushScala.js');
		wp_enqueue_script('syntaxhighlighter-brush-sql', $plugin_url . '/highlighter/scripts/shBrushSql.js');
		wp_enqueue_script('syntaxhighlighter-brush-vb', $plugin_url . '/highlighter/scripts/shBrushVb.js');
		wp_enqueue_script('syntaxhighlighter-brush-xml', $plugin_url . '/highlighter/scripts/shBrushXml.js');
	}
	
}

function external_footer() {
	
	global $plugin_url;
	
?>
	<script class="javascript">
	SyntaxHighlighter.config.clipboardSwf = '<?php echo $plugin_url; ?>/highlighter/Scripts/clipboard.swf';
	SyntaxHighlighter.all();
	</script>
<?php	
}

function external_file_error($msg)
{
	return '<div class="error">' . $msg . '</div>';
}

function external($atts, $content = null)
{
	
	global $formats;
	
	$debug = '';
	
	extract(shortcode_atts(array(
		'file' => $content,
		'format' => 'auto',
		'start' => null,
		'end' => null
	), $atts));
	
	if($file == null)
	{
		return external_file_error("No file specified");
	}
	
	$data = @file($file);
	
	if($data == false) {
		return external_file_error("The specified file could not be found");
	}
	
	// if the format has not been set, is set to auto, or is not in the set of allowed formats, we'll have to work it out!
	if($format == 'auto' or !in_array($format, $formats))
	{
		if(is_file($file)) { // we can assume the file is a file on disk
			$file_type = pathinfo($file, PATHINFO_EXTENSION);
			switch($file_type)
			{
				case 'htm':
				case 'html':
					$format = 'xml';
				break;
				case 'php':
				case 'phps':
					$format = 'php';
				break;
				case 'sql':
					$format = 'sql';
				break;
				case 'css':
					$format = 'css';
				case 'js':
					$format = 'js';
				break;
				case 'pl':
					$format = 'pl';
				break;
				case 'py':
					$format = 'py';
				break;
				default:
					$format = 'plain';
				break;
			}
		} else { // assume the file is a url
			
			if(function_exists('curl_init'))
			{
				$ch = curl_init($file);
				curl_setopt($ch, CURLOPT_MUTE, 1);
				curl_setopt($ch, CURLOPT_VERBOSE, 0);
				curl_setopt($ch, CURLOPT_NOBODY, 1);
				curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
				curl_exec($ch);
				$mime = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
				curl_close($ch);
			} else {
				$mime = null;
			}
			
			switch($mime)
			{
				case 'text/html':
				case 'text/xml':
				case 'text/xhtml':
				case 'application/xhtml+xml':
					$format = 'xml';
				break;
				case 'application/javascript':
				case 'application/json':
					$format = 'js';
				break;
				case 'text/css':
					$format = 'css';
				break;
				default:
					$format = 'plain';
				break;
			}
		}
	}
	
	// $format = ($format != 'auto') ? $format : pathinfo($file, PATHINFO_EXTENSION);
	
	$output = '';
	
	$start = (is_numeric($start) and $start != null) ? $start : 1;
	$end = (is_numeric($end) and $end != null) ? $end : count($data);
	
	foreach($data AS $row => $line) {
		if($row >= $start-1 and $row < $end)
		{
			$output .= $line;
		}
	}
	
	$output = htmlentities($output);
		
	return "<pre name=\"code\" class=\"brush: $format; first-line: $start\">$output</pre>";
}

add_action('wp_print_styles', 'external_header');
add_action('init', 'external_scripts');
add_action('wp_footer', 'external_footer');
add_shortcode('external', 'external');

?>